package org.dict.client;

import org.dict.server.*;
import java.awt.*;
import java.io.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.awt.datatransfer.*;
import org.dict.kernel.*;
/**
 * Insert the type's description here.
 * Creation date: (28.02.2002 23:18:11)
 * @author:
 */
public class DictSwingView implements Runnable, ListSelectionListener, ActionListener {
  Clipboard clipboard = java.awt.Toolkit.getDefaultToolkit().getSystemClipboard();
  String value = getClipboardText(clipboard);
  IDictEngine engine;
  JFrame fFrame;
  JTextArea fDisplay;
  JCheckBox mode;
  JTextField inputField;
  JTextField statusField;
  JList matches;
  JComboBox db, fontChoice, sizeChoice;
  String[] history = new String[20];
  int pos = 0;
  //int fontSize = Integer.getInteger("fontSize", 12).intValue();
  int delay = Integer.getInteger("delay", 20).intValue();

    class ML extends MouseAdapter {
        public void mouseClicked(MouseEvent e) {
            if (e.getClickCount() == 2) {
                DictSwingView.this.lookup(DictSwingView.this.getSelectedText(), false);
            }
        }
    }
    class WL extends WindowAdapter {
        public void windowActivated(WindowEvent e) {
	        DictSwingView.this.lookup();
        }
        public void windowClosing(WindowEvent e) {
	        System.exit(0);
        }
    }
    class History implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            DictSwingView.this.showHistory();
        }
    }
    class IL implements ItemListener {
	    public void itemStateChanged(ItemEvent e) {
		    DictSwingView.this.setFont();
	    }
    }
/**
 * ClipboardTest constructor comment.
 */
public DictSwingView(String cfg) {
	this(org.dict.server.DatabaseFactory.getEngine(cfg));
}
/**
 * ClipboardTest constructor comment.
 */
public DictSwingView(IDictEngine e) {
	super();
	this.engine = e;
	initialize();
}
public void actionPerformed(java.awt.event.ActionEvent e) {
	lookup(getInputField().getText(), false);
}
private static String getClipboardText(Clipboard c) {
    Transferable content = c.getContents(null);
    if (content == null) {
        return null;
    }
    try {
        String s = (String) content.getTransferData(DataFlavor.stringFlavor);
		return s;
    } catch (Exception e) {
	    return null;
    }
}
protected JTextArea getDisplay() {
	return fDisplay;
}
protected JFrame getFrame() {
	return fFrame;
}
protected JTextField getInputField() {
	return inputField;
}
private String getSelectedText() {
    return getDisplay().getSelectedText();
}
private void initialize() {
	fFrame = new JFrame("Intelligent dictionary lookup");
	fFrame.setSize(600, 400);
	fDisplay = new JTextArea();
	fDisplay.setBackground(Color.white);
	fDisplay.setForeground(Color.black);
	fDisplay.addMouseListener(new ML());
	JScrollPane scrollPane = new JScrollPane(fDisplay);
	inputField = new JTextField(15);
	inputField.addActionListener(this);
	statusField = new JTextField(25);
	statusField.setEditable(false);
	matches = new JList(new DefaultListModel());
	matches.setPrototypeCellValue("123456789012345");
	matches.addListSelectionListener(this);
	//matches.addListSelectionListener(new LSL());
	JScrollPane scrollList = new JScrollPane(matches);
	//scrollList.setPreferredSize(new Dimension(100, 400));
	JButton prev = new JButton("History");
	prev.addActionListener(new History());
	mode = new JCheckBox("Auto", false);
	db = new JComboBox();
	for (int i = 0; i < engine.getDatabases().length; i++){
		String name = engine.getDatabases()[i].getName();
		if (name.length() > 20) name = name.substring(0,17)+"...";
		db.addItem(name);
	}
	db.addItem("Any database");
	fontChoice = new JComboBox(new String[]{"Arial", "Verdana", "Courier New"});
	fontChoice.setEditable(true);
	fontChoice.addItemListener(new IL());
	sizeChoice = new JComboBox(new String[]{"10", "12", "15"});
	sizeChoice.setEditable(true);
	sizeChoice.addItemListener(new IL());
	JPanel north = new JPanel();
	north.setBackground(Color.lightGray);
	north.add(mode);
	north.add(db);
	north.add(prev);
	north.add(inputField);
	JPanel south = new JPanel();
	south.add(statusField);
	south.add(fontChoice);
	south.add(sizeChoice);
	JPanel jp = new JPanel(new BorderLayout());
	jp.add(north, "North");
	jp.add(scrollPane, "Center");
	jp.add(south, "South");
	jp.add(scrollList, "West");
	fFrame.getContentPane().add(jp);
	fFrame.addWindowListener(new WL());
	db.setSelectedItem("Any database");
	fontChoice.setSelectedItem("Verdana");
	sizeChoice.setSelectedItem("12");
	fFrame.setVisible(true);
}
private void lookup() {
    String s = getClipboardText(clipboard);
    if (s != null && !s.equals(this.value)) {
        this.value = s;
        lookup(s, false);
    }
}
protected void lookup(String s, boolean append) {
    if (s == null || s.length() == 0 || s.length() > 25) {
        return;
    }
    history[pos++ % history.length] = s;
    statusField.setText("Lookup: "+s);
    getFrame().toFront();
    String dbID = "*";
    int dbIndex = db.getSelectedIndex();
    if (dbIndex < engine.getDatabases().length) {
	    dbID = engine.getDatabases()[dbIndex].getID();
    }
    IAnswer[] a = engine.defineMatch(dbID, s, null, true, IDatabase.STRATEGY_NEAR);
    StringWriter w = new StringWriter();
    PrintWriter out = new PrintWriter(w);
    IRequest req = new SimpleRequest("", "word="+s);
    try {
        org.dict.server.PlainPrinter.printAnswers(engine, req, a, false, out);
        out.flush();
    } catch (Exception e) {
        e.printStackTrace(out);
    }
    showMatches(a);
    if (append) {
        getDisplay().setText(getDisplay().getText()+w.toString());
    } else {
        getDisplay().setText(w.toString());
    }
}
private void loop() {
    while (true) {
        try {
            Thread.sleep(delay);
        } catch (InterruptedException e) {
        }
        if (mode.isSelected()) {
            lookup();
        }
    }
}
public static void main(String[] args) {
	if (args.length < 1) {
		System.out.println("Usage: java ...DictSwingView cfg");
		System.exit(0);
	}
    java.util.Vector v = new java.util.Vector();
    for (int i = 0; i < args.length; i++){
    	IDatabase[] arr = DatabaseFactory.getEngine(args[i]).getDatabases();
    	for (int k = 0; k < arr.length; k++){
    		v.addElement(arr[k]);
    	}
    }
    IDatabase[] dbs = new IDatabase[v.size()];
    v.copyInto(dbs);
    DictEngine engine = new DictEngine();
    engine.setDatabases(dbs);
	DictSwingView runner = new DictSwingView(engine);
	Thread th = new Thread(runner);
	th.start();
	if (Boolean.getBoolean("startServer")) {
		org.dict.server.JDictd.listen(args);
	}
}
public void run() {
    loop();
}
private void setFont() {
	String face = (String) fontChoice.getSelectedItem();
	String size = (String) sizeChoice.getSelectedItem();
	Font fnt = new Font(face, Font.PLAIN, Integer.parseInt(size));
    getDisplay().setFont(fnt);
    getInputField().setFont(fnt);
    matches.setFont(fnt);
}
private void showHistory() {
	DefaultListModel model = (DefaultListModel) matches.getModel();
	model.removeAllElements();
    for (int i = 0; i < history.length; i++){
    	if (history[i] != null) model.addElement(history[i]);
    }
}
private void showMatches(IAnswer[] a) {
	DefaultListModel model = (DefaultListModel) matches.getModel();
	model.removeAllElements();
    for (int i = 0; i < a.length; i++){
    	INeighbor[] n = a[i].getMatches();
    	for (int k = 0; k < n.length; k++) {
    		model.addElement(n[k].getKey());
    	}
    	model.addElement(""); // separator
    }
}
 /** 
   * Called whenever the value of the selection changes.
   * @param e the event that characterizes the change.
   */
public void valueChanged(javax.swing.event.ListSelectionEvent e) {
	lookup((String)matches.getSelectedValue(), false);
}
}
