package org.dict.client;

import java.net.Socket;
import org.dict.server.*;
import java.awt.*;
import java.io.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.awt.datatransfer.*;
import org.dict.kernel.*;
/**
 * Insert the type's description here.
 * Creation date: (28.02.2002 23:18:11)
 * @author:
 */
public class JDictGUI implements Runnable, ActionListener {
	java.util.Properties fProperties;
    Clipboard clipboard = java.awt.Toolkit.getDefaultToolkit().getSystemClipboard();
    String value = getClipboardText(clipboard);
    JFrame fFrame;
    JTextArea fDisplay;
    JCheckBox mode;
    JTextField inputField;
    JComboBox db, fontChoice, sizeChoice;
    private java.util.Vector hist = new java.util.Vector(11);
    private int histCurrent;
    int delay = Integer.getInteger("delay", 20).intValue();

    class ML extends MouseAdapter {
        public void mouseClicked(MouseEvent e) {
            if (e.getClickCount() == 2) {
                JDictGUI.this.lookupSelected();
            }
        }
    }
    class WL extends WindowAdapter {
        public void windowActivated(WindowEvent e) {
            JDictGUI.this.lookup();
        }
        public void windowClosing(WindowEvent e) {
            System.exit(0);
        }
    }
    class GoBack implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            JDictGUI.this.goBack();
        }
    }
    class GoForward implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            JDictGUI.this.goForward();
        }
    }
    class LookupSelected implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            JDictGUI.this.lookupSelected();;
        }
    }
    class IL implements ItemListener {
        public void itemStateChanged(ItemEvent e) {
            JDictGUI.this.setFont();
        }
    }
/**
 * Insert the method's description here.
 * Creation date: (14.03.2002 21:21:02)
 */
public JDictGUI(String cfg) {
	super();
	initialize(cfg);
}
public void actionPerformed(java.awt.event.ActionEvent e) {
	lookup(getInputField().getText(), true);
}
void addHist(String s) {
    histCurrent++;
    if (histCurrent >= hist.size()) {
        histCurrent = hist.size();
        hist.addElement(s);
    } else {
        hist.setElementAt(s, histCurrent);
        hist.setSize(histCurrent + 1);
    }
}
public void addKeyListeners(java.util.Hashtable h) {
	for (java.util.Enumeration e = h.keys(); e.hasMoreElements();) {
		try {
			Component c = (Component) e.nextElement();
			KeyListener l = (KeyListener) h.get(c);
			c.addKeyListener(l);
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}
}
public void appendText(String res, String db) {
    getDisplay().setText(getDisplay().getText()+"\n----------\n"+res);
}
private static String getClipboardText(Clipboard c) {
    Transferable content = c.getContents(null);
    if (content == null) {
        return null;
    }
    try {
        String s = (String) content.getTransferData(DataFlavor.stringFlavor);
		return s;
    } catch (Exception e) {
	    return null;
    }
}
protected JTextArea getDisplay() {
	return fDisplay;
}
protected JFrame getFrame() {
	return fFrame;
}
protected JTextField getInputField() {
	return inputField;
}
private String getSelectedText() {
    return getDisplay().getSelectedText();
}
public void goBack() {
    if (hist.size() > 1 && histCurrent > 0) {
        histCurrent--;
        String s = (String) hist.elementAt(histCurrent);
        lookup(s, false);
    }
}
public void goForward() {
    if (hist.size() > 1 && histCurrent < hist.size() - 1) {
        histCurrent++;
        String s = (String) hist.elementAt(histCurrent);
        lookup(s, false);
    }
}
private void initialize() {
	fFrame = new JFrame("Intelligent dictionary lookup");
	fFrame.setSize(600, 400);
	fDisplay = new JTextArea();
	fDisplay.setBackground(Color.white);
	fDisplay.setForeground(Color.black);
	fDisplay.addMouseListener(new ML());
	JScrollPane scrollPane = new JScrollPane(fDisplay);
	inputField = new JTextField(15);
	inputField.addActionListener(this);
	JButton prev = new JButton("Back");
	prev.setToolTipText("Go to last lookup");
	prev.addActionListener(new GoBack());
	JButton forward = new JButton("Forward");
	prev.setToolTipText("Go to next lookup");
	forward.addActionListener(new GoForward());
	JButton sel = new JButton("Selection");
	sel.setToolTipText("Lookup selected word/phrase");
	sel.addActionListener(new LookupSelected());
	mode = new JCheckBox("Auto", false);
	mode.setToolTipText("Lookup automatically when clipboard content changed");
	java.util.Vector v = initServers();
	db = new JComboBox(v);
	db.setEditable(true);
	fontChoice = new JComboBox(new String[]{"Arial", "Verdana", "Courier New"});
	fontChoice.setEditable(true);
	fontChoice.addItemListener(new IL());
	sizeChoice = new JComboBox(new String[]{"10", "12", "15"});
	sizeChoice.setEditable(true);
	sizeChoice.addItemListener(new IL());
	JPanel north = new JPanel();
	north.setBackground(Color.lightGray);
	north.add(prev);
	north.add(db);
	north.add(inputField);
	north.add(forward);
	JPanel south = new JPanel();
	south.add(mode);
	south.add(sel);
	south.add(fontChoice);
	south.add(sizeChoice);
	JPanel jp = new JPanel(new BorderLayout());
	jp.add(north, "North");
	jp.add(scrollPane, "Center");
	jp.add(south, "South");
	fFrame.getContentPane().add(jp);
	fFrame.addWindowListener(new WL());
	fontChoice.setSelectedItem("Verdana");
	sizeChoice.setSelectedItem("12");
	fFrame.setVisible(true);
}
private void initialize(String cfg) {
	fProperties = System.getProperties();
	if (cfg == null) {
		return;
	}
	try {
		InputStream in = new FileInputStream(cfg);
		fProperties.load(in);
		in.close();
	} catch (Throwable t) {
	}
	initialize();
}
private java.util.Vector initServers() {
	java.util.Vector v = new java.util.Vector(5);
	String s = fProperties.getProperty("servers", "");
	java.util.StringTokenizer st = new java.util.StringTokenizer(s, ";");
	while (st.hasMoreElements()) {
		v.addElement(st.nextToken());
	}
	return v;
}
private void lookup() {
    String s = getClipboardText(clipboard);
    if (s != null && !s.equals(this.value)) {
        this.value = s;
        lookup(s, true);
    }
}
protected void lookup(String s, boolean addHistory) {
    if (s == null || s.length() == 0 || s.length() > 25) {
        return;
    }
    getInputField().setText(s);
    getDisplay().setText(s);
    getFrame().toFront();
    new Thread(this).start();
    if (addHistory) {
	    addHist(s);
    }
}
private void lookupSelected() {
    lookup(getSelectedText(), true);
}
public void loop() {
    while (true) {
        try {
            Thread.sleep(delay);
        } catch (InterruptedException e) {
        }
        if (mode.isSelected()) {
            lookup();
        }
    }
}
public static void main(String[] args) {
	String serverCfg = null, cfg = null;
	for (int i = 0; i < args.length-1; i++){
		if (args[i].equalsIgnoreCase("-s")) {
			serverCfg = args[++i];
		} else if (args[i].equalsIgnoreCase("-c")) {
			cfg = args[++i];
		}
	}
	if (serverCfg != null) {
		JDictd.listen(new String[] {serverCfg});
	}
	new JDictGUI(cfg).loop();
}
String request(String url) {
    BufferedReader in = null;
    StringBuffer sb = new StringBuffer();
    try {
	    java.net.URL u = new java.net.URL(url);
	    java.net.URLConnection conn = u.openConnection();
	    String enc = conn.getContentEncoding();
	    if (enc == null) {
		    enc = "latin1";
	    }
        in = new BufferedReader(new InputStreamReader(conn.getInputStream(), enc));
        String line;
        while ((line = in.readLine()) != null) {
            sb.append(line).append('\n');
        }
        in.close();
    } catch (Exception e) {
        sb.append(e);
    }
    return sb.toString();
}
String request(String word, String host, int port, String[] dbs) {
    Socket s = null;
    PrintWriter out = null;
    BufferedReader in = null;
    StringBuffer sb = new StringBuffer();
    try {
        s = new Socket(host, port);
        out =
            new PrintWriter(new OutputStreamWriter(s.getOutputStream(), "UTF-8"), true);
        in = new BufferedReader(new InputStreamReader(s.getInputStream(), "UTF-8"));
        String line = in.readLine(); // Server banner
        //out.println("client \"Java DICT client\"");
        for (int i = 0; i < dbs.length; i++) {
            out.println("define " + dbs[i] + " \"" + word + "\"");
            line = in.readLine();
            if (line.startsWith("552") && !line.startsWith("550")) {
                out.println("match " + dbs[i] + " prefix " + word);
            }
        }
        out.println("quit");
        while ((line = in.readLine()) != null) {
            sb.append(line).append('\n');
        }
        out.close();
        in.close();
        s.close();
    } catch (Exception e) {
        sb.append(e);
    }
    return sb.toString();
}
public void run() {
	String word = getInputField().getText();
    String database = (String) db.getSelectedItem();
    String res = null;
    if (database.startsWith("http:")) {
	    res = request(database+word);
    } else {
	    ServerInfo si = ServerInfo.getInstance(database);
    	res = request(word, si.host, si.port, si.dbs);
    }
    appendText(res, database);
}
private void setFont() {
	String face = (String) fontChoice.getSelectedItem();
	String size = (String) sizeChoice.getSelectedItem();
	Font fnt = new Font(face, Font.PLAIN, Integer.parseInt(size));
    getDisplay().setFont(fnt);
    getInputField().setFont(fnt);
}
}
