package org.dict.kernel;

import java.util.Vector;
import java.io.*;
//import com.sun.java.util.collections.*;
/**
 * Insert the type's description here.
 * Creation date: (28.07.01 21:54:33)
 * @author: Administrator
 */
public abstract class Database implements IDatabase {
	private KeyList fIndex;
	private String fDatafile;
	private String fName;
	private String fID;
	private String fEncoding = "utf-8";
	private IComparator fComparator;
	private IMorphAnalyzer fMorphAnalyzer;
	private String fSourceLanguage;
	private String fTargetLanguage;
	private static int MAX_MATCHES = Integer.getInteger("matches", 10).intValue();
/**
 * Database constructor comment.
 */
public Database() {
	super();
}
int compare(IKey k1, IKey k2) {
	return getComparator().compare(k1, k2);
}
/**
 * define method comment.
 */
public String define(String word) {
	return defineMatch(word, true, STRATEGY_NONE).getDefinition();
}
/**
 * define method comment.
 */
public IAnswer defineMatch(int pos, boolean define, int strat) {
	String ret = null;
	KeyList kl = getIndex();
	if (pos < 0 || pos >= kl.size()) {
		pos = Math.abs(pos) % kl.size();
	}
	kl.startUp();
	String word = ((IKey)kl.get(pos)).getKey();
	if (define && pos >= 0) {
		byte[] b = readDefinitions(pos);
		try {
			ret = new String(b, getEncoding());
		} catch (Exception e) {
			ret = new String(b);
		}
	}
	INeighbor[] matches = null;
	if (strat == STRATEGY_EXACT) {
		matches = getExactMatches(word, pos);
	}
	if (strat == STRATEGY_NEAR) {
		matches = getNeighbors(word, pos, MAX_MATCHES);
	}
	if (strat == STRATEGY_PREFIX) {
		matches = getPrefixMatches(word, pos, MAX_MATCHES);
	}
	kl.shutDown();
	return new Answer(this, word, pos, ret, matches);
}
/**
 * define method comment.
 */
public IAnswer defineMatch(String word, boolean define, int strat) {
	String ret = null;
	KeyList kl = getIndex();
	IKey k = new Key(word, null, null);
	kl.startUp();
	int pos = search(k, define);
	if (define && pos >= 0) {
		byte[] b = readDefinitions(pos);
		try {
			ret = new String(b, getEncoding());
		} catch (Exception e) {
			ret = new String(b);
		}
	}
	INeighbor[] matches = null;
	if (strat == STRATEGY_EXACT) {
		matches = getExactMatches(word, pos);
	}
	if (strat == STRATEGY_NEAR) {
		matches = getNeighbors(word, pos, MAX_MATCHES);
	}
	if (strat == STRATEGY_PREFIX) {
		matches = getPrefixMatches(word, pos, MAX_MATCHES);
	}
	kl.shutDown();
	return new Answer(this, word, pos, ret, matches);
}
/**
 * Insert the method's description here.
 * Creation date: (11.08.01 10:09:50)
 * @return org.dict.kernel.IComparator
 */
public IComparator getComparator() {
	return fComparator;
}
/**
 * Insert the method's description here.
 * Creation date: (28.07.01 22:01:41)
 * @return java.lang.String
 */
public java.lang.String getDatafile() {
	return fDatafile;
}
/**
 * Insert the method's description here.
 * Creation date: (28.07.01 22:01:41)
 * @return java.lang.String
 */
public java.lang.String getDescription() {
	return define("00-database-info");
}
/**
 * Insert the method's description here.
 * Creation date: (03.09.01 21:55:54)
 * @return java.lang.String
 */
public java.lang.String getEncoding() {
	return fEncoding;
}
INeighbor[] getExactMatches(String word, int pos) {
	if (pos < 0) {
		return new Neighbor[0];
	} else {
		return new Neighbor[] {new Neighbor(word, pos)};
	}
}
/**
 * Insert the method's description here.
 * Creation date: (7/30/01 6:39:21 PM)
 * @return java.lang.String
 */
public java.lang.String getID() {
	return fID;
}
/**
 * Insert the method's description here.
 * Creation date: (28.07.01 22:01:41)
 * @return org.dict.IKeyList
 */
public KeyList getIndex() {
	return fIndex;
}
/**
 * Insert the method's description here.
 * Creation date: (02.09.01 12:46:21)
 * @return org.dict.kernel.IMorphAnalyzer
 */
public IMorphAnalyzer getMorphAnalyzer() {
	return fMorphAnalyzer;
}
/**
 * Insert the method's description here.
 * Creation date: (29.07.01 22:00:15)
 * @return java.lang.String
 */
public java.lang.String getName() {
	return fName;
}
INeighbor[] getNeighbors(String word, int pos, int max) {
	KeyList kl = getIndex();
	if (pos < 0) {
		pos = -pos - 1;
	}
	int beg = Math.max(0, pos - (max/2));
	int end = Math.min(kl.size()-1, beg+max);
	INeighbor[] ret = new INeighbor[end-beg+1];
	for (int i = beg; i <= end; i++){
		IKey k = (IKey) kl.get(i);
		ret[i-beg] = new Neighbor(k.getKey(), i);
	}
	return ret;
}
INeighbor[] getPrefixMatches(String word, int pos, int max) {
	KeyList kl = getIndex();
	if (pos < 0) {
		pos = -pos - 1;
	}
	Vector ls = new Vector(max);
	String lword = word.toLowerCase();
	boolean match = false;
	do {
		if (pos >= kl.size() || ls.size() >= max) {
			break;
		}
		Key k = (Key) kl.get(pos);
		String w = k.getKey();
		if (w.toLowerCase().startsWith(lword)) {
			ls.addElement(new Neighbor(w, pos));
			match = true;
		} else {
			match = false;
		}
		pos++;
	} while (match);
	INeighbor[] ret = new INeighbor[ls.size()];
	ls.copyInto(ret);
	return ret;
}
/**
 * Insert the method's description here.
 * Creation date: (10.03.2002 18:53:59)
 * @return java.lang.String
 */
public java.lang.String getSourceLanguage() {
	return fSourceLanguage;
}
/**
 * Insert the method's description here.
 * Creation date: (10.03.2002 18:53:59)
 * @return java.lang.String
 */
public java.lang.String getTargetLanguage() {
	return fTargetLanguage;
}
/**
 * match method comment.
 */
public INeighbor[] match(String word, int strategy) {
	return defineMatch(word, false, strategy).getMatches();
}
byte[] readAll(int start, int end) {
	IKey k = (IKey) getIndex().get(start);
	long off = BASE64Converter.parse(k.getOffset());
	long len = 0L;
	for (int i = start; i <= end; i++) {
		k = (IKey) getIndex().get(i);
		len += BASE64Converter.parse(k.getLength());
	}
	try {
		return readData(off, len);
	} catch (IOException e) {
		ByteArrayOutputStream w = new ByteArrayOutputStream();
		e.printStackTrace(new PrintWriter(w));
		return w.toByteArray();
	}
}
protected abstract byte[] readData(long offset, long len) throws java.io.IOException;
byte[] readDefinitions(int pos) {
	KeyList kl = getIndex();
	IKey k = (IKey) kl.get(pos);
	int start = pos, end = pos;
	while (start > 0) {
		IKey k2 = (IKey) kl.get(start - 1);
		if (compare(k2, k) == 0) {
			start--;
		} else {
			break;
		}
	}
	while (end < kl.size() - 1) {
		IKey k2 = (IKey) kl.get(end + 1);
		if (compare(k2, k) == 0) {
			end++;
		} else {
			break;
		}
	}
	return readAll(start, end);
}
public int search(IKey k, IMorphAnalyzer ma) {
	int pos = ListUtil.search(getIndex(), k, getComparator());
	if (pos >= 0 || ma == null) return pos;
	int range = 500;
	int beg = Math.max(0, -pos-range+50);
	int end = Math.min(beg+range, getIndex().size()-1);
	String[] bases = ma.getPossibleBases(k.getKey());
	for (int i = 0; i < bases.length; i++){
		IKey k2 = new Key(bases[i], null, null);
		int p2 = ListUtil.search(getIndex(), k2, getComparator(), beg, end);
		if ( p2 >= 0) return p2;
	}
	return pos;
}
public int search(IKey k, boolean useMorph) {
	if (useMorph) {
		return search(k, getMorphAnalyzer());
	} else {
		return search(k, null);
	}
}
/**
 * Insert the method's description here.
 * Creation date: (11.08.01 10:09:50)
 * @param newComparator org.dict.kernel.IComparator
 */
public void setComparator(IComparator newComparator) {
	fComparator = newComparator;
}
/**
 * Insert the method's description here.
 * Creation date: (28.07.01 22:01:41)
 * @param newDatafile java.lang.String
 */
public void setDatafile(java.lang.String newDatafile) {
	fDatafile = newDatafile;
}
/**
 * Insert the method's description here.
 * Creation date: (03.09.01 21:55:54)
 * @param newEncoding java.lang.String
 */
public void setEncoding(java.lang.String newEncoding) {
	fEncoding = newEncoding;
}
/**
 * Insert the method's description here.
 * Creation date: (7/30/01 6:39:21 PM)
 * @param newID java.lang.String
 */
public void setID(java.lang.String newID) {
	fID = newID;
}
/**
 * Insert the method's description here.
 * Creation date: (28.07.01 22:01:41)
 * @param newIndex org.dict.KeyList
 */
public void setIndex(KeyList newIndex) {
	fIndex = newIndex;
}
/**
 * Insert the method's description here.
 * Creation date: (02.09.01 12:46:21)
 * @param newMorphAnalyzer org.dict.kernel.IMorphAnalyzer
 */
public void setMorphAnalyzer(IMorphAnalyzer newMorphAnalyzer) {
	fMorphAnalyzer = newMorphAnalyzer;
}
/**
 * Insert the method's description here.
 * Creation date: (29.07.01 22:00:15)
 * @param newName java.lang.String
 */
public void setName(java.lang.String newName) {
	fName = newName;
}
/**
 * Insert the method's description here.
 * Creation date: (10.03.2002 18:53:59)
 * @param newSourceLanguage java.lang.String
 */
public void setSourceLanguage(java.lang.String newSourceLanguage) {
	fSourceLanguage = newSourceLanguage;
}
/**
 * Insert the method's description here.
 * Creation date: (10.03.2002 18:53:59)
 * @param newTargetLanguage java.lang.String
 */
public void setTargetLanguage(java.lang.String newTargetLanguage) {
	fTargetLanguage = newTargetLanguage;
}
}
