package org.dict.kernel;

import java.util.*;
/**
 * Insert the type's description here.
 * Creation date: (21.4.2002 12:11:31)
 * @author: 
 */
public class VirtualDatabase implements IDatabase {
	IDatabase fIn, fOut;
	private String fName;
	private String fID;
/**
 * VirtualDatabase constructor comment.
 */
public VirtualDatabase() {
	super();
}
/**
 * VirtualDatabase constructor comment.
 */
public VirtualDatabase(String id, String name, IDatabase in, IDatabase out) {
	super();
	fID = id;
	fName = name;
	fIn = in;
	fOut = out;
}
void addDefinition(String word, boolean def, int strat, Vector v, Vector done) {
	if (done.contains(word) || v.size() > 5) return;
	done.addElement(word);
	IAnswer a = getOut().defineMatch(word, def, strat);
	if (a.getDefinition() != null) {
		v.addElement(a);
	} else {
		StringTokenizer st = new StringTokenizer(word, " ");
		while (st.hasMoreTokens()) {
			String tok = st.nextToken();
			addDefinition(tok, def, strat, v, done);
		}
	}
}
void append(StringBuffer sb, String s) {
	sb.append(s);
}
/**
 * define method comment.
 */
public String define(String word) {
	return defineMatch(word, true, STRATEGY_NONE).getDefinition();
}
IAnswer[] defineAll(String[] arr, boolean def, int strat) {
	Vector v = new Vector();
	Vector done = new Vector();
	done.addElement("of");
	done.addElement("to");
	for (int i = 0; i < arr.length; i++){
		addDefinition(arr[i], def, strat, v, done);
	}
	IAnswer[] ret = new IAnswer[v.size()];
	v.copyInto(ret);
	return ret;
}
/**
 * This command will look up the word with specified index in the database
 */
public IAnswer defineMatch(int pos, boolean define, int strategy) {
	IAnswer in = getIn().defineMatch(pos, define, strategy);
	String s = in.getDefinition();
	if (s == null) return new Answer(this, in.getKey(), pos, s, in.getMatches());
	StringBuffer sb = new StringBuffer();
	append(sb, s);
	sb.append("\n\n");
	String[] arr = getTranslations(s, getID());
	IAnswer[] answers = defineAll(arr, define, strategy);
	for (int i = 0; i < answers.length; i++){
		append(sb, answers[i].getDefinition());
	}
	INeighbor[] matches = in.getMatches();
	return new Answer(this, in.getKey(), pos, sb.toString(), matches);
}
/**
 * This command will look up the specified word in the database
 */
public IAnswer defineMatch(String word, boolean define, int strategy) {
	IAnswer in = getIn().defineMatch(word, define, strategy);
	String s = in.getDefinition();
	if (s == null) return new Answer(this, word, in.getPosition(), s, in.getMatches());
	StringBuffer sb = new StringBuffer();
	append(sb, s);
	sb.append("\n\n");
	String[] arr = getTranslations(s, getID());
	IAnswer[] answers = defineAll(arr, define, strategy);
	for (int i = 0; i < answers.length; i++){
		append(sb, answers[i].getDefinition());
	}
	int pos = in.getPosition();
	INeighbor[] matches = in.getMatches();
	return new Answer(this, word, pos, sb.toString(), matches);
}
/**
 * Show the database info
 */
public String getDescription() {
	return getIn().getDescription()+getOut().getDescription();
}
/**
 * Insert the method's description here.
 * Creation date: (21.4.2002 12:15:32)
 * @return java.lang.String
 */
public java.lang.String getID() {
	return fID;
}
/**
 * Insert the method's description here.
 * Creation date: (21.4.2002 12:12:46)
 * @return org.dict.kernel.IDatabase
 */
public IDatabase getIn() {
	return fIn;
}
/**
 * Insert the method's description here.
 * Creation date: (21.4.2002 12:15:32)
 * @return java.lang.String
 */
public java.lang.String getName() {
	return fName;
}
/**
 * Insert the method's description here.
 * Creation date: (21.4.2002 12:12:46)
 * @return org.dict.kernel.IDatabase
 */
public IDatabase getOut() {
	return fOut;
}
/**
 * getSourceLanguage method comment.
 */
public String getSourceLanguage() {
	return getIn().getSourceLanguage();
}
/**
 * getTargetLanguage method comment.
 */
public String getTargetLanguage() {
	return getOut().getTargetLanguage();
}
public static String[] getTranslations(String data, String id) {
	if ("vg".equals(id)) return getTranslationsVE(data);
	return getTranslationsGE(data);
}
public static String[] getTranslationsGE(String data) {
	int pos = data.indexOf('\n');
	String s = data.substring(pos+1).trim();
	java.util.StringTokenizer st = new java.util.StringTokenizer(s, ",;{}\n");
	java.util.Vector v = new java.util.Vector(5);
	while (st.hasMoreTokens()) {
		v.addElement(st.nextToken().trim());
	}
	String[] ret = new String[v.size()];
	v.copyInto(ret);
	return ret;
}
public static String[] getTranslationsVE(String data) {
	java.util.Vector v = new java.util.Vector(5);
	StringTokenizer st = new StringTokenizer(data, "\n");
	while (st.hasMoreTokens()) {
		String s = st.nextToken().trim();
		if (s.startsWith("-")) {
			StringTokenizer st2 = new StringTokenizer(s.substring(1).trim(), ",;");
			while (st2.hasMoreTokens()) {
				v.addElement(st2.nextToken());
			}
		}
	}
	String[] ret = new String[v.size()];
	v.copyInto(ret);
	return ret;
}
/**
 * match method comment.
 */
public INeighbor[] match(String word, int strategy) {
	return defineMatch(word, false, strategy).getMatches();
}
/**
 * Insert the method's description here.
 * Creation date: (21.4.2002 12:15:32)
 * @param newID java.lang.String
 */
public void setID(java.lang.String newID) {
	fID = newID;
}
/**
 * Insert the method's description here.
 * Creation date: (21.4.2002 12:12:46)
 * @param newIn org.dict.kernel.IDatabase
 */
public void setIn(IDatabase newIn) {
	fIn = newIn;
}
/**
 * Insert the method's description here.
 * Creation date: (21.4.2002 12:15:32)
 * @param newName java.lang.String
 */
public void setName(java.lang.String newName) {
	fName = newName;
}
/**
 * Insert the method's description here.
 * Creation date: (21.4.2002 12:12:46)
 * @param newOut org.dict.kernel.IDatabase
 */
public void setOut(IDatabase newOut) {
	fOut = newOut;
}
}
