package org.dict.server;

import java.net.*;
import java.io.*;
import java.util.*;
import org.dict.kernel.*;

public class DictHTTPConnection implements Runnable {
    private IDictEngine fEngine;
    private Socket theConnection;
    public DictHTTPConnection(IDictEngine e, Socket s) {
	    fEngine = e;
	    theConnection = s;
    }
	public void run() 
	{
		String method = "";
		String uri = "";
		String params = "";
		String version = "";
		int fContentLength = 0;
		
		try 
		{
			//PrintWriter os = new PrintWriter(new OutputStreamWriter(theConnection.getOutputStream(), "UTF8"));
			InputStream input = theConnection.getInputStream();
			BufferedReader is = 
				new BufferedReader(new InputStreamReader(input, "UTF-8"));
			do
			{
				String request = is.readLine();
				if(request == null || request.length() == 0)
					break;
				StringTokenizer tokenizer = new StringTokenizer(request, " ");
				if(!tokenizer.hasMoreTokens())
					break;
				String first = tokenizer.nextToken();
				if(first.equals("GET"))
				{
					method = "GET";
					String query = tokenizer.nextToken();
    				int idx = query.indexOf('?');
   					if (idx == -1) {
        				processGet(query, theConnection.getOutputStream());
        				theConnection.close();
        				return;
    				} else {
	    				uri = query.substring(0, idx);
	    				params = query.substring(idx+1);
    				}
				}
				else
				if(first.equals("POST"))
				{
					method = "POST";
					uri = tokenizer.nextToken();
				}
				else
				if(request.toUpperCase().startsWith("CONTENT-LENGTH:"))
					fContentLength = Integer.parseInt(tokenizer.nextToken());
			}
			while(true);
			if (method.equals("POST")) {
				char[] b = new char[fContentLength];
				is.read(b, 0, fContentLength);
				params = new String(b);
			}
			OutputStream os = theConnection.getOutputStream();
			serve(os, uri, params);
			os.flush();
		}
		catch(Throwable e)
		{
			e.toString();
		}
		// After the data have been sent, close the connection
		try 
		{
			theConnection.close();
		}
		catch (IOException e) 
		{
		}
	}

private String getHead(String start, String end) {
    return start
        + "\nAllow: GET, HEAD, POST\nMIME-Version: 1.0\n"
        + "Server: "
        + "JDictd 1.2"
        + "\n"
        + end
        + "\r\n\r\n";
}

protected String getOutputEncoding(IRequest req) {
	return "UTF-8";
}

protected String getRemote() {
	return theConnection.getInetAddress().getHostName();
}

private void processError(int code, OutputStream os) {
    String message = null;
    switch (code) {
        case 400 :
            message =
                getHead("HTTP/1.0 400 Bad Request", "")
                    + "<HTML><BODY><H1>Bad Request</H1>"
                    + "The server could not understand this request.<P></BODY></HTML>";
            break;

        case 404 :
            message =
                getHead("HTTP/1.0 404 Not Found", "")
                    + "<HTML><BODY><H1>Not Found</H1>"
                    + "The server could not find this file.<P></BODY></HTML>";
            break;

        case 403 :
            message =
                getHead("HTTP/1.0 403 Forbidden", "")
                    + "<HTML><BODY><H1>Forbidden</H1>"
                    + "Access is not allowed.<P></BODY></HTML>";
            break;
    }
    try {
        os.write(message.getBytes("latin1"));
        os.flush();
        //os.close();
    } catch (Exception e) {
        //e.printStackTrace();
    }
}

private void processGet(String name, OutputStream os) {
    try {
        if (name.endsWith("/"))
            name = name + "index.html";
        if (name.indexOf("..") != -1) {
            processError(403, os);
            return;
        }
        BufferedInputStream file = null;
        String header;
        String root = System.getProperty("ROOT", ".");
        try {
            file = new BufferedInputStream(new FileInputStream(new File(root, name)));
            int len = file.available();
            header =
                getHead(
                    "HTTP/1.0 200 OK",
                    "Content-Type: text/html\n" + "Content-Length: " + len);
        } catch (IOException _ex) {
            processError(404, os);
            return;
        }
        os.write(header.getBytes("latin1"));
        byte[] b = new byte[1024];
        int len;
        while ((len = file.read(b)) > 0) {
	        os.write(b, 0, len);
        }
        os.flush();
        //os.close();
    } catch (Exception e) {
        //e.printStackTrace();
    }
}

private void serve(OutputStream os, String uri, String params) throws IOException {
    IRequest req = new SimpleRequest(uri, params);
    serve(os, req);
    Logger.getInstance().log(getRemote()+" "+params);
}

protected void serve(OutputStream os, IRequest req) throws IOException {
	String enc = getOutputEncoding(req);
	PrintWriter out = new PrintWriter(new OutputStreamWriter(os, enc));
    try {
		out.print("HTTP/1.0 200 OK\r\nContent-type: text/html; charset="+enc+"\r\n\r\n");
		IAnswer[] arr = fEngine.lookup(req);
		HTMLPrinter.printAnswers(fEngine, req, arr, true, out);
		out.flush();
    } catch (Throwable t) {
		out.println("<pre>");
        t.printStackTrace(out);
		out.println("</pre>");
		out.flush();
    }
}
}
